@/usr/local/rsi/idl_5.5/lib/gaussfit.pro
pro sim_deconv, cubename, prefix, lcenoff, lwidoff, lintoff

; re-create a datacube with dimensions (x1,x2,x3), lambda-axis is x2. 
; 
; input parameters: 
; cubename: 	string specifying which data cube:
;		"helium2", "serts", "neon8", "carbon4", "oxygen6"
; prefix:	prefix for output files, e.g. "he2_"
; lcenoff:	offset from true line center, 
;		ideally 0, but choose other values to investigate 
;		results for ill-guessed 4th projection
; lwidoff;	same for line width
; lintoff;	same for line intensity (useless, since we normalize)
;
; output:
; 	(1) .sav files containing data cube and backprojected data cube
;	(2) a file called prefix+"totnegs" containing for every slice of the 
;	    cube the slicenumber, quadratical average of residuals 
;	    of applied gaussian to line profiles, and final number 
; 	    of pixels carrying negative values
;	(3) a file called "tmp" holding for each 10th iteration step 
;	    the iteration step and total number of pixels having neg. 
;	    values. Used for convergence plot. Overwritten with each 
;	    new slice at the moment.
;
; AT 19-09-2003

start_time=systime()

tmp = cleanup()		; cleanup open files etc. from previous runs

theta=[-45,0,45,90]	; array holding angles of projections (for MOSES: -45,0,45,90)
Nt=n_elements(theta)
overl90=1		; if 1, replace theta=90 projection with gaussian
graphics=1		; if 0, surpress 2dim graphics
linegraph=1		; switch graphics for line profile check
fitgraph=1		; switch graphics for gaussfit check
backcheck=0		; switch graphics for background determination check
Niter=1000		; maximum number of iterations 
if fitgraph eq 1 then window, 1
if linegraph eq 1 then window, 2, xsize=500,ysize=400
if backcheck eq 1 then window, 3, xpos=0, ypos=200, xsize=400, ysize=400


cube = get_cube(cubename, "cube")		; get cube
;cube = despike(cube)				; remove cosmics
dim=size(cube)
padd_1=(dim[2]-dim[1])/2			; needed later to re-do 
padd_2=padd_1+dim[1]-1				;   zeropadding. Assume Nx>Nl
cube_back = get_background(cubename, cube, backcheck)      ; get background
cube = zeropad(cube)				; zero-pad to square x,y
cube_back = zeropad(cube_back) 			; zero-pad the background
cube = (cube-cube_back)                     	; correct for background

; get number of slices to handle, and new cube dimensions
dim=size(cube)
Nl=dim[1]
Nx=dim[2]
Ny=dim[3]
backcube = fltarr(Nl,Nx,Ny)			; will hold backprojected cube

; calculate masks
if (n_elements(theta) eq 4) then begin
	if (contains(theta,-45) eq 1) and (contains(theta,0) eq 1) and $
	    (contains(theta, 45) eq 1) and (contains(theta,90) eq 1) then begin
		print, "standard masks for -45,0,45,90 used"
		mask = make_4masks(Nl)	
		mask_case="standard"	; flag for case theta=[-45,0,45,90]
	endif
endif else begin
	print, "costum masks used"
	x1=Nl mod 2 & x2 = Nx mod 2
	if ((x1 eq 1) or (x2 eq 1)) then begin
		print, "custom projection angles (i.e. other then the MOSES ones) "
		print, "only work correctly for odd size of x- and lambda-axes!"
		return
	endif
	mask = make_masks(theta, cube)	; only works if Nl,Nx are odd numbers
	mask_case="costum"		; flag for cases other than above
endelse

;keep number of negatives for each slice in file
;openw, totnegsfile, prefix+"totnegs", /get_lun
openw, 4, prefix+"totnegs"
;keep average line profile properties
openw, 6, prefix+"lineparameters_av.dat"

for sl=0,Ny-1 do begin	; loop over slices, covering each lambda,x plane
	;print, "working on ", cubename, " slice ",sl, " out of ",Ny-1
	image = reform(cube[*,*,sl])
	image = rotate(image,1)	; wavelength on y-axis f
	; get gaussian parameters of total profile
	lpar = get_lineparameters(cubename, image, fitgraph) 
	printf, 6, sl, padd_1, padd_2, lpar[0],lpar[1]
	; make overlappograms
	if (mask_case eq "standard") then begin
		overl = overlappograms4(image, mask)
	endif 
	if (mask_case eq "costum") then begin
		overl = overlappograms(image, mask, theta)
	endif
	; replace theta=90 overlappogram with gaussians, if required
	i90 = where(theta eq 90)   
	if (i90 ne -1) and (overl90 eq 1) and (lpar[0] ne -1) then begin
		; keep original average profile
		org_profile = total(reform(overl[*,*,i90]), 1)/Nl 
        	x = findgen(Nl)  ; lambda-axis for fitted gaussian
        	m90 = fltarr(Nl) ; will hold fitted gaussian
		; vary the gaussian parameters to simulate bad guess
		lcen = lpar[0]+lcenoff
		lwid = lpar[1]+lwidoff
		lint = lpar[2]+lintoff
		; put in fitted gaussian and stretch it to 2dim array
		m90 = m90+lint * exp(-0.5*((x-lcen)/lwid)^2) ;+lpar[3]
        	overl[*,*,i90] = replicate(1.0,Nl) # m90
		; normalize
		tmp1 = total(org_profile)
		tmp2 = total(total(reform(overl[*,*,i90]), 1)/Nl)
		overl[*,*,i90] = overl[*,*,i90]*tmp1/tmp2
		; compare original and new fitted average profile
		new_profile = total(reform(overl[*,*,i90]), 1)/Nl
                diff = org_profile-new_profile
                profdiff = sqrt(total(diff^2)/n_elements(diff)) ; quality number
                if linegraph eq 1 then begin
			; comparison plot for line profiles if desired
                       	wset,2 
			!p.thick=1. & !p.charsize=1.
			plotmax = max([org_profile,new_profile,diff])
			plotmin = min([org_profile,new_profile,diff])
			titlestr="average profiles: slice "+$
			    strtrim(string(sl),2)+" out of "+$
			    strtrim(string(Ny-1),2)
                       	    plot, org_profile, $
				  yrange=[plotmin,plotmax], $
                                  title=titlestr
                       	    oplot, new_profile, line=2
			    oplot, diff, line=3
			legend,['average','fit','diff'], linestyle=[0,2,3]
			wait, .5
                endif
	endif	; end of if-block replacing theta=90 with average profile
	; combine overlappograms
	image_fft = combine_overlap(image, mask, overl)
	; backproject
	backprj = float(fft(image_fft,/inverse))
	; iterate for non-negativity
	backprj = iter_backprj(Niter,backprj,image_fft,mask,"tmp",graphics,0,sl,Ny)
	; add slice to reconstructed cube
	backcube[*,*,sl] = rotate(backprj,3)
	; make a convergence plot, and retain last number of totnegs
	endnegs = graph_convergence("tmp", prefix, 2) ; if 2, then plot to .eps
	print,  "slice, gaussfit residual, endnegs: ",sl, profdiff, endnegs
	printf, 4, sl,profdiff,endnegs
endfor

;close, totnegsfile
close, 4
close, 6

; add background back on
;backcube = backcube+cube_back

; redo zeropadding
cube=cube[padd_1:padd_2,*,*]
backcube=backcube[padd_1:padd_2,*,*]
; save the original cube and backprojected cube
cubefile = prefix+"cube.sav"
resultfile = prefix+"backcube.sav"
save, cube, filename=cubefile
save, backcube, filename=resultfile

; running time
end_time = systime()
print
print, "started "+start_time
print, "ended   "+end_time
min1=minutes(start_time)
min2=minutes(end_time)
dur=min2-min1
if dur lt 1 then begin
	t=dur*60
	un=" seconds"
endif
if (dur ge 1) and (dur lt 60) then begin
	t=dur
	un=" minutes"
endif
if (dur ge 60) then begin
	t=dur/60.
	un=" hours"
endif
print, 'duration '+strtrim(string(t),2)+un


end
